package org.mentalog.util;

public class Benchmarker {

	private final static Benchmarker instance = new Benchmarker();

	private long time;
	private long count = 0;
	private long totalTime = 0;
	private long minTime = Long.MAX_VALUE;
	private long maxTime = Long.MIN_VALUE;

	public final static Benchmarker instance() {
		return instance;
	}

	public void mark() {
		time = System.nanoTime();
	}

	public void measure() {
		count++;
		long lastTime = System.nanoTime() - time;
		totalTime += lastTime;
		minTime = Math.min(minTime, lastTime);
		maxTime = Math.max(maxTime, lastTime);
	}

	private final double avg() {
		if (count == 0) return 0;
		double avg = ((double) totalTime / (double) count);
		double rounded = Math.round(avg * 100D) / 100D;
		return rounded;
	}

	public String results() {
		StringBuilder sb = new StringBuilder(128);
		sb.append("totalLogs=").append(count);
		sb.append(", totalTime=").append(totalTime);
		sb.append(", minTime=").append(minTime);
		sb.append(", maxTime=").append(maxTime);
		sb.append(", avgTime=").append(String.valueOf(avg()));
		return sb.toString();
	}
}