package org.mentalog.util;

import java.util.Calendar;
import java.util.TimeZone;

public class CheapDateTimeFormatter {
	
	private final char[] time = "HH:mm:ss.SSS".toCharArray();
	
	private long lastTime = 0;
	private long offset;
	private TimeZone tz;
	
	private char[] day, month, year;
	private long nextDay;
	
	public CheapDateTimeFormatter(TimeZone tz) {
		this.offset = getTimeZoneOffset(tz);
		this.tz = tz;
		parseCurrentDate();
	}
	
	public CheapDateTimeFormatter() {
		this(TimeZone.getDefault());
	}
	
	private String p(int i) {
		if (i < 10) return "0" + i;
		return String.valueOf(i);
	}
	
	public char[] getDay(long now) { 
		checkIfNextDay(now);
		return day;
	}
	
	public char[] getMonth(long now) {
		checkIfNextDay(now);
		return month;
	}
	
	public char[] getYear(long now) { 
		checkIfNextDay(now);
		return year;
		}
	
	private void parseCurrentDate() {

		Calendar cal = Calendar.getInstance();
		cal.setTimeZone(tz);
		
		day = p(cal.get(Calendar.DAY_OF_MONTH)).toCharArray();
		month = p(cal.get(Calendar.MONTH) + 1).toCharArray();
		year = p(cal.get(Calendar.YEAR)).toCharArray();
		
		markNextStartDate(cal);
	}
	
	private void markNextStartDate(Calendar cal) {
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		
		cal.add(Calendar.DAY_OF_MONTH, 1);
		
		this.nextDay = cal.getTimeInMillis();
	}
	
	private final void checkIfNextDay(long now) {
		if (now >= nextDay) {
			parseCurrentDate();
		}
	}
	
	public void changeTimeZone(TimeZone tz) {
		this.offset = getTimeZoneOffset(tz);
		this.tz = tz;
		parseCurrentDate();
	}
	
	private static long getTimeZoneOffset(TimeZone tz) {
		
        Calendar cal = Calendar.getInstance();
        
        return tz.getOffset( cal.get( Calendar.ERA ),
                               cal.get( Calendar.YEAR ),
                               cal.get( Calendar.MONTH ),
                               cal.get( Calendar.DAY_OF_MONTH ),
                               cal.get( Calendar.DAY_OF_WEEK ),
                               cal.get( Calendar.MILLISECOND ) );
	}
	
    public char[] formatTime(long timeInMillis) {

        long now = timeInMillis + offset;
        
        if( now == lastTime )  return time;

        lastTime = now;

        time[ 11 ] = (char) ( '0' + now % 10 );
        now /= 10;
        time[ 10 ] = (char) ( '0' + now % 10 );
        now /= 10;
        time[ 9 ] = (char) ( '0' + now % 10 );
        now /= 10;

        int sec = (int) ( now % 60 );
        time[ 7 ] = (char) ( '0' + sec % 10 );
        sec /= 10;
        time[ 6 ] = (char) ( '0' + sec % 10 );
        now /= 60;

        int min = (int) ( now % 60 );
        time[ 4 ] = (char) ( '0' + min % 10 );
        min /= 10;
        time[ 3 ] = (char) ( '0' + min % 10 );
        now /= 60;

        int hr = (int) ( now % 24 );
        time[ 1 ] = (char) ( '0' + hr % 10 );
        hr /= 10;
        time[ 0 ] = (char) ( '0' + hr % 10 );
        
        return time;
    }
    
    public void formatTimeTo(StringBuffer sb, long timeInMillis) {
    	
    	char[] time = formatTime(timeInMillis);
    	
    	for(int i = 0; i < time.length; i++) {
    		
    		sb.append(time[i]);
    	}
    }
    
    public static void main(String[] args) {
    	
    	CheapDateTimeFormatter formatter = new CheapDateTimeFormatter();
    	
    	long now = System.currentTimeMillis();
    
    	System.out.println("F: " + new String(formatter.formatTime(now)));
    	
    	System.out.println("DAY: " + new String(formatter.getDay(now)));
    	System.out.println("MONTH: " + new String(formatter.getMonth(now)));
    	System.out.println("YEAR: " + new String(formatter.getYear(now)));
    	

    }

}